import sys
import logging
import traceback
from socket import socket
sys.path.append('../')
from log_profile.logger_server import SERVER
from log_profile.logger_client import CLIENT


if (sys.argv[0].split('/')[-1]).lower() == f'{SERVER}.py':
    logger = logging.getLogger(SERVER)
if (sys.argv[0].split('/')[-1]).lower() == f'{CLIENT}.py':
    logger = logging.getLogger(f'{CLIENT}')


def log_decorator(func):
    def wrapper(*args, **kwargs):
        res = func(*args, **kwargs)
        logger.debug(f'Вызвана функция {func.__name__} с параметрами {args}, {kwargs}. '
                     f'Вызов из модуля {func.__module__}. '
                     f'Вызов из функции {traceback.format_stack()[0].strip().split()[-1]}.'
                     )
        return res
    return wrapper


def login_required(func):
    def checker(*args, **kwargs):
        from server.core import MessageProcessor
        from common.variables import ACTION, PRESENCE
        if isinstance(args[0], MessageProcessor):
            found = False
            for arg in args:
                if isinstance(arg, socket):
                    for client in args[0].names:
                        if args[0].names[client] == arg:
                            found = True

            for arg in args:
                if isinstance(arg, dict):
                    if ACTION in arg and arg[ACTION] == PRESENCE:
                        found = True
            if not found:
                raise TypeError
        return func(*args, **kwargs)

    return checker



import os
from os import path
import logging.handlers


CLIENT = 'client'


def base_format(module):
    return f'%(levelname)s - %(asctime)s - {module} - %(message)s'


def log_dir(value):
    # return f'{path.join(path.dirname(__file__))}/../logs/{value}_log/{value}.log'
    return f'{os.getcwd()}/../logs/{value}_log/{value}.log'

logger = logging.getLogger(CLIENT)

logger_handler = logging.handlers.TimedRotatingFileHandler(
    log_dir(CLIENT), encoding='utf-8', when='midnight', interval=1
)
logger_handler.setLevel(logging.DEBUG)
logger_handler.setFormatter(logging.Formatter(base_format(CLIENT)))

logger.addHandler(logger_handler)
logger.setLevel(logging.DEBUG)

if __name__ == '__main__':
    logger.critical('Critical_client')
    logger.error('Error_client')
    logger.warning('Warning_client')
    logger.info('Info_client')
    logger.debug('Debug_client')



import logging
from log_profile.logger_server import SERVER
from ipaddress import ip_address

logger = logging.getLogger(SERVER)


class Port:
    def __set__(self, instance, value):
        if not 1023 < value < 65536:
            print('Так нельзя!')
            sys.exit(1)
            logger.critical(
                        f'Попытка запуска сервера с указанием неподходящего порта '
                        f'{value}. Допустимые адреса с 1024 до 65535.')
        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name


# class Address:
#     def __set__(self, instance, value):
#         print(value)
#         try:
#             ip_address(value)
#             instance.__dict__[self.name] = value
#         except:
#             print('так тоже нельзя')
#
#     def __set_name__(self, owner, name):
#         self.name = name



import sys
import os
from unittest import TestCase, main
sys.path.append(os.path.join(os.getcwd(), '..'))
from common.variables import ACTION, PRESENCE, RESPONSE, ERROR, TIME, USER, ACCOUNT_NAME
from server import process_client_message


class TestServer(TestCase):
    error_dict = {
        RESPONSE: 400,
        ERROR: 'Bad Request'
    }
    ok_dict = {RESPONSE: 200}

    def test_no_action(self):
        self.assertEqual(process_client_message(
                {TIME: '1.1', USER: {ACCOUNT_NAME: 'Oleg'}}), self.error_dict
        )

    def test_incorrect_action(self):
        self.assertEqual(process_client_message(
            {ACTION: 'Error', TIME: 1.1, USER: {ACCOUNT_NAME: 'Oleg'}}), self.error_dict
        )

    def test_no_time(self):
        self.assertEqual(process_client_message(
            {ACTION: PRESENCE, USER: {ACCOUNT_NAME: 'Oleg'}}), self.error_dict
        )

    def test_no_user(self):
        self.assertEqual(process_client_message(
            {ACTION: PRESENCE, TIME: 1.1}), self.error_dict
        )

    def test_incorrect_user(self):
        self.assertEqual(process_client_message(
            {ACTION: PRESENCE, TIME: 1.1, USER: {ACCOUNT_NAME: 'User'}}), self.error_dict
        )

    def test_ok_check(self):
        self.assertNotEqual(process_client_message(
            {ACTION: PRESENCE, TIME: 1.1, USER: {ACCOUNT_NAME: 'Oleg'}}), self.error_dict
        )


if __name__ == '__main__':
    main()



from unittest import TestCase, main
import sys
import os
sys.path.append(os.path.join(os.getcwd(), '..'))
from common.variables import ACTION, PRESENCE, RESPONSE, ERROR, TIME, USER, ACCOUNT_NAME
from client import create_presence, process_ans


class TestClient(TestCase):
    def test_def_presence(self):
        test = create_presence()
        test[TIME] = 1.2
        self.assertEqual(test, {
        ACTION: PRESENCE,
        TIME: 1.2,
        USER: {
            ACCOUNT_NAME: 'Oleg'
        }
    })

    def test_200_ans(self):
        self.assertEqual(process_ans({RESPONSE: 200}), '200 OK')

    def test_400_ans(self):
        self.assertEqual(process_ans({RESPONSE: 400, ERROR: 'Bad Request'}), '400 Bad Request')

    def test_no_response(self):
        self.assertRaises(ValueError, process_ans, {ERROR: 'Bad Request'})


if __name__ == '__main__':
    main()




from PyQt5.QtWidgets import QMainWindow, QAction, qApp, QApplication, QLabel, QTableView
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtCore import QTimer
from .stat_window import StatWindow
from .setting_window import SettingWindow
from .add_user import RegisterUser
from .remove_user import DelUserDialog


class MainWindow(QMainWindow):
    def __init__(self, database, server, config):
        super().__init__()

        self.database = database

        self.server_thread = server
        self.config = config


        self.exit_action = QAction('Выход', self)
        self.exit_action.setShortcut('Ctrl+Q')
        self.exit_action.triggered.connect(qApp.quit)

        self.refresh_btn = QAction('Обновить', self)
        self.register_btn = QAction('Регистрация пользователя', self)
        self.remove_btn = QAction('Удаление пользователя', self)
        self.history_btn = QAction('История', self)
        self.setting_btn = QAction('Настройки сервера', self)
        self.help_btn = QAction('Помощь', self)

        self.statusBar()
        menu_bar = self.menuBar()
        file_menu_1 = menu_bar.addMenu('&File')
        file_menu_1.addAction(self.refresh_btn)
        file_menu_1.addAction(self.register_btn)
        file_menu_1.addAction(self.remove_btn)
        file_menu_1.addAction(self.history_btn)
        file_menu_1.addAction(self.exit_action)

        file_menu_2 = menu_bar.addMenu('&Settings')
        file_menu_2.addAction(self.setting_btn)

        file_menu_3 = menu_bar.addMenu('&Help')
        file_menu_3.addAction(self.help_btn)
        self.statusBar().showMessage('Server Working')

        # self.toolbar = self.addToolBar('MainBar')
        # self.toolbar.addAction(self.exit_action)
        # self.toolbar.addAction(self.refresh_btn)
        # self.toolbar.addAction(self.history_btn)
        # self.toolbar.addAction(self.setting_btn)
        # self.toolbar.addAction(self.register_btn)
        # self.toolbar.addAction(self.remove_btn)

        self.setFixedSize(800, 600)
        self.setWindowTitle('Messaging Server alpha release')

        self.label = QLabel('Список подключённых клиентов:', self)
        self.label.setFixedSize(240, 15)
        self.label.move(10, 25)

        self.active_clients_table = QTableView(self)
        self.active_clients_table.move(10, 45)
        self.active_clients_table.setFixedSize(780, 400)

        self.timer = QTimer()
        self.timer.timeout.connect(self.create_users_model)
        self.timer.start(1000)

        self.refresh_btn.triggered.connect(self.create_users_model)
        self.history_btn.triggered.connect(self.show_statistics)
        self.setting_btn.triggered.connect(self.server_config)
        self.register_btn.triggered.connect(self.reg_user)
        self.remove_btn.triggered.connect(self.rem_user)

        self.show()

    def create_users_model(self):
        list_users = self.database.active_users_list()
        list = QStandardItemModel()
        list.setHorizontalHeaderLabels(
            ['Имя Клиента', 'IP Адрес', 'Порт', 'Время подключения'])
        for row in list_users:
            user, ip, port, time = row
            user = QStandardItem(user)
            user.setEditable(False)
            ip = QStandardItem(ip)
            ip.setEditable(False)
            port = QStandardItem(str(port))
            port.setEditable(False)
            time = QStandardItem(str(time.replace(microsecond=0)))
            time.setEditable(False)
            list.appendRow([user, ip, port, time])
        self.active_clients_table.setModel(list)
        self.active_clients_table.resizeColumnsToContents()
        self.active_clients_table.resizeRowsToContents()

    def show_statistics(self):
        global stat_window
        stat_window = StatWindow(self.database)
        stat_window.show()

    def server_config(self):
        global config_window
        config_window = SettingWindow(self.config)

    def reg_user(self):
        global reg_window
        reg_window = RegisterUser(self.database, self.server_thread)
        reg_window.show()

    def rem_user(self):
        global rem_window
        rem_window = DelUserDialog(self.database, self.server_thread)
        rem_window.show()



import os
from os import path
import logging.handlers


SERVER = 'server'


def base_format(module):
    return f'%(levelname)s - %(asctime)s - {module} - %(message)s'


def log_dir(value):
    # return f'{path.join(path.dirname(__file__))}/../logs/{value}_log/{value}.log'
    return f'{os.getcwd()}/../logs/{value}_log/{value}.log'

logger = logging.getLogger(SERVER)

logger_handler = logging.handlers.TimedRotatingFileHandler(
    log_dir(SERVER),
    encoding='utf-8', when='midnight', interval=1
)

logger_handler.setLevel(logging.DEBUG)
logger_handler.setFormatter(logging.Formatter(base_format(SERVER)))

logger.addHandler(logger_handler)
logger.setLevel(logging.DEBUG)


if __name__ == '__main__':
    logger.critical('Critical_server')
    logger.error('Error_server')
    logger.warning('Warning_server')
    logger.info('Info_server')
    logger.debug('Debug_server')



from setuptools import setup, find_packages

setup(name="salmin_server",
      version="0.0.2",
      description="salmin_server",
      author="Oleg Salmin",
      author_email="olegsalmin74@gmail.com",
      packages=find_packages(),
      install_requires=['PyQt5', 'sqlalchemy', 'pycryptodome', 'pycryptodomex']
      )




import uuid
from sqlalchemy.types import BINARY
from sqlalchemy import types


class UUID(types.TypeDecorator):
    impl = BINARY
    cache_ok = False

    def __init__(self):
        self.impl.length = 16
        types.TypeDecorator.__init__(self, length=self.impl.length)

    def process_bind_param(self, value, dialect=None):
        if value and isinstance(value, uuid.UUID):
            return value.bytes
        elif value and not isinstance(value, uuid.UUID):
            raise ValueError(f'value %s is not a valid uuid.UUID {value}')
        else:
            return None

    def process_result_value(self, value, dialect=None):
        if value:
            return uuid.UUID(bytes=value)
        else:
            return None

    def is_mutable(self):
        return False


import os
from os import path
import sys
import argparse
import configparser
from PyQt5.QtCore import Qt
from server.core import MessageProcessor
from server.database import ServerStorage
from log_profile.logger_server import SERVER
from common.variables import *
from common.decor import log_decorator
from PyQt5.QtWidgets import QApplication
from server.main_window import MainWindow

logger = logging.getLogger(SERVER)


@log_decorator
def arg_parser(default_address, default_port):
    '''Парсер аргументов коммандной строки.'''
    parser = argparse.ArgumentParser()
    parser.add_argument('-a', default=default_address, nargs='?')
    parser.add_argument('-p', default=default_port, type=int, nargs='?')
    namespace = parser.parse_args(sys.argv[1:])
    listen_address = namespace.a
    listen_port = namespace.p
    return listen_address, listen_port


@log_decorator
def config_load():
    '''Парсер конфигурационного ini файла.'''
    config = configparser.ConfigParser()
    # dir_path = path.dirname(path.realpath(__file__))
    dir_path = os.getcwd()
    config.read(f"{dir_path}/{'server.ini'}")
    if 'SETTINGS' in config:
        return config
    else:
        config.add_section('SETTINGS')
        config.set('SETTINGS', 'Default_port', str(DEFAULT_PORT))
        config.set('SETTINGS', 'Listen_Address', '')
        config.set('SETTINGS', 'Database_file', 'server_database.db3')
        return config


@log_decorator
def main():
    '''Основная функция'''
    config_file = config_load()

    listen_address, listen_port = arg_parser(
        config_file['SETTINGS']['Listen_Address'],
        config_file['SETTINGS']['Default_port']
    )

    database = ServerStorage(
        config_file['SETTINGS']['Database_file'].split('\\')[-1]
    )

    server = MessageProcessor(listen_address, listen_port, database)
    server.daemon = True
    server.start()

    server_app = QApplication(sys.argv)
    server_app.setAttribute(Qt.AA_DisableWindowContextHelpButton)
    main_window = MainWindow(database, server, config_file)

    server_app.exec_()

    server.running = False


if __name__ == '__main__':
    main()
